﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;
using Petzold.SoundSynthesis;

namespace SimpleKeyboardSynthesizer
{
    public partial class KeyboardKey : UserControl
    {
        const int CXWHITE = 24; // 48;
        const int CYWHITE = 96; // 192;
        const int CXBLACK = 12; // 24;
        const int CYBLACK = 60; // 120;
        const int INDENT = CXBLACK / 2;

        Point[] ptsCF = { new Point(0, 0), new Point(0, CYWHITE), 
                          new Point(CXWHITE, CYWHITE), new Point(CXWHITE, CYBLACK),
                          new Point(CXWHITE - INDENT, CYBLACK),
                          new Point(CXWHITE - INDENT, 0) };

        Point[] ptsDGA = { new Point(INDENT, 0), new Point(INDENT, CYBLACK), 
                           new Point(0, CYBLACK), new Point(0, CYWHITE),
                           new Point(CXWHITE, CYWHITE), new Point(CXWHITE, CYBLACK),
                           new Point(CXWHITE - INDENT, CYBLACK), new Point(CXWHITE - INDENT, 0) };

        Point[] ptsEB =  { new Point(INDENT, 0), new Point(INDENT, CYBLACK), 
                           new Point(0, CYBLACK), new Point(0, CYWHITE),
                           new Point(CXWHITE, CYWHITE), new Point(CXWHITE, 0) };

        Point[] ptsBlack = { new Point(-INDENT, 0), new Point(-INDENT, CYBLACK), 
                             new Point(INDENT, CYBLACK), new Point(INDENT, 0) };

        Brush whiteBrush = new SolidColorBrush(Colors.White);
        Brush blackBrush = new SolidColorBrush(Colors.Black);
        Brush highlightBrush = new SolidColorBrush(Colors.Gray);

        Polygon poly;
        Brush savedBrush;

        public static readonly DependencyProperty HighlightProperty =
            DependencyProperty.Register("Highlight",
                typeof(bool),
                typeof(KeyboardKey),
                new PropertyMetadata(false, OnHighlightChanged));

        public KeyboardKey(Key key, Note note)
        {
            InitializeComponent();

            poly = new Polygon();
            poly.Stroke = blackBrush;
            poly.Fill = whiteBrush;
            Point[] ptArray = null;
            Panel destination = LayoutRoot;

            TextBlock txtblk = new TextBlock();
            txtblk.Width = CXWHITE;
            txtblk.TextAlignment = TextAlignment.Center;
            txtblk.VerticalAlignment = VerticalAlignment.Bottom;
            txtblk.Margin = new Thickness(0, 0, 0, 10);
            txtblk.Foreground = blackBrush;
            txtblk.Text = key.ToString();

            if (txtblk.Text.Length == 2)
                txtblk.Text = txtblk.Text.Substring(1);

            switch (note)
            {
                case Note.C:
                case Note.F:
                    ptArray = ptsCF;
                    break;

                case Note.D:
                case Note.G:
                case Note.A:
                    ptArray = ptsDGA;
                    break;

                case Note.E:
                case Note.B:
                    ptArray = ptsEB;
                    break;

                default:
                    ptArray = ptsBlack;
                    poly.Fill = blackBrush;
                    txtblk.Width = CXBLACK;
                    txtblk.Foreground = whiteBrush;
                    Canvas.SetLeft(txtblk, -INDENT);
                    Canvas.SetTop(txtblk, CYBLACK - 20);
                   
                    destination = canvas;
                    break;
            }

            foreach (Point pt in ptArray)
                poly.Points.Add(pt);

            destination.Children.Add(poly);
            destination.Children.Add(txtblk);
        }

        public bool Highlight
        {
            set { SetValue(HighlightProperty, value); }
            get { return (bool)GetValue(HighlightProperty); }
        }

        static void OnHighlightChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args)
        {
            (obj as KeyboardKey).OnHighlightChanged(args);
        }

        void OnHighlightChanged(DependencyPropertyChangedEventArgs args)
        {
            if ((bool)args.NewValue)
            {
                savedBrush = poly.Fill;
                poly.Fill = highlightBrush;
            }
            else
            {
                poly.Fill = savedBrush;
            }
        }
    }
}
